/*
   WebSocketServer.ino

    Created on: 22.05.2015

*/
//The following constants were borrowed from the library
//These are our button constants
#define PSB_SELECT      0x0001
#define PSB_L3          0x0002
#define PSB_R3          0x0004
#define PSB_START       0x0008
#define PSB_PAD_UP      0x0010
#define PSB_PAD_RIGHT   0x0020
#define PSB_PAD_DOWN    0x0040
#define PSB_PAD_LEFT    0x0080
#define PSB_L2          0x0100
#define PSB_R2          0x0200
#define PSB_L1          0x0400
#define PSB_R1          0x0800
#define PSB_GREEN       0x1000
#define PSB_RED         0x2000
#define PSB_BLUE        0x4000
#define PSB_PINK        0x8000
#define PSB_TRIANGLE    0x1000
#define PSB_CIRCLE      0x2000
#define PSB_CROSS       0x4000
#define PSB_SQUARE      0x8000

//These are analog buttons
#define PSAB_PAD_RIGHT    9
#define PSAB_PAD_UP      11
#define PSAB_PAD_DOWN    12
#define PSAB_PAD_LEFT    10
#define PSAB_L2          19
#define PSAB_R2          20
#define PSAB_L1          17
#define PSAB_R1          18
#define PSAB_GREEN       13
#define PSAB_RED         14
#define PSAB_BLUE        15
#define PSAB_PINK        16
#define PSAB_TRIANGLE    13
#define PSAB_CIRCLE      14
#define PSAB_CROSS       15
#define PSAB_SQUARE      16

//These are stick values
#define PSS_RX 5
#define PSS_RY 6
#define PSS_LX 7
#define PSS_LY 8

#include <Arduino.h>

#include <ESP8266WiFi.h>
//#include <ESP8266WiFiMulti.h>
#include <WebSocketsServer.h>
#include <Hash.h>


#define APSSID "AP"
#define APPSK  "min8char"


/* Set these to your desired credentials. */
const char *ssid = APSSID;
const char *password = APPSK;

unsigned long nextHeartBeatMillis = 0;
unsigned long nextSerialWriteMillis = 0;
uint16_t buttons = 0xFFFF;

unsigned char GamePadData[21] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

WebSocketsServer webSocket = WebSocketsServer(81);

boolean ButtonOn(uint16_t button) { //This function borrowed from library
  return ((~(uint16_t)buttons & (uint16_t)button) > 0);
}

void webSocketEvent(uint8_t num, WStype_t type, uint8_t * payload, size_t lenght)
{
  switch (type)
  {
    case WStype_DISCONNECTED:
      Serial.printf("[%u] Disconnected!\n", num);
      break;
    case WStype_CONNECTED:
      {
        IPAddress ip = webSocket.remoteIP(num);
        Serial.printf("[%u] Connected froms %d.%d.%d.%d url: %s\n", num, ip[0], ip[1], ip[2], ip[3], payload);
        memset(GamePadData, 255, 5);

        // send message to client
        webSocket.sendTXT(num, "Connected");
      }
      break;
    case WStype_TEXT:
      if (lenght == 21) {

        memcpy(GamePadData, payload,  21);

        for (int i = 0; i < 21; i++) {
          if(GamePadData[i] < 0x10) {Serial.print("0");}
          Serial.print(GamePadData[i], HEX);
         // Serial.print("::");
        }
        Serial.println();
        buttons =  (uint16_t)(payload[4] << 8) + payload[3];
        analogWrite(D1, GamePadData[PSAB_BLUE]); //Set PWM value of D1 to reflect the pressure applied to the BLUE 'X' on the controller
        //hexdump(payload, lenght); //used for debug

      }
      break;
    case WStype_BIN:
      {
        Serial.printf("[%u] get binary lenght: %u\n", num, lenght);
        hexdump(payload, lenght);

        // send message to client
        // webSocket.sendBIN(num, payload, lenght);
      }
      break;
  }

}

void setup()
{
  // Serial.begin(921600);
  Serial.begin(500000);
  //Serial.begin(115200);
 // Serial.setDebugOutput(true);

  Serial.println();
  Serial.println();

  Serial.print("Configuring access point...");
  /* You can remove the password parameter if you want the AP to be open. */
  WiFi.disconnect();
  WiFi.persistent(false);
  WiFi.mode(WIFI_AP);
  Serial.println(WiFi.softAP(ssid, password) ? "Ready" : "Failed!");
  //WiFi.softAP(ssid, password);
  IPAddress myIP = WiFi.softAPIP();
  Serial.print("AP IP address: ");
  Serial.println(myIP);

  webSocket.begin();
  webSocket.onEvent(webSocketEvent);
  pinMode(D1, OUTPUT);
  analogWriteRange(255); //Set PWM range 0 - 255
  memset(GamePadData, 255, 5); //Initial first 5 bytes to 0xFF . Important to set Buttons bytes 3&4 to FF so no false trigger on startup
}

void loop()
{
  webSocket.loop();
  if (millis() > nextHeartBeatMillis )
  {
    String mils = String(millis(), DEC);
    webSocket.broadcastTXT(mils);//String(millis(), DEC));
    nextHeartBeatMillis = millis() + 2000;
  }
 //The following can be useful for debugging or using in an alternate way
  /*
    if (ButtonOn(PSB_PAD_RIGHT))
    { //will be TRUE as long as button is pressed
      Serial.print("PSAB_PAD_RIGHT held this hard: ");
      Serial.println(GamePadData[PSAB_PAD_RIGHT], DEC);
    }

    if (ButtonOn(PSB_PAD_LEFT))
    { //will be TRUE as long as button is pressed
      Serial.print("PSAB_PAD_LEFT held this hard: ");
      Serial.println(GamePadData[PSAB_PAD_LEFT], DEC);
    }

    if (ButtonOn(PSB_PAD_UP))
    { //will be TRUE as long as button is pressed
      Serial.print("PSAB_PAD_UP held this hard: ");
      Serial.println(GamePadData[PSAB_PAD_UP], DEC);
    }

    if (ButtonOn(PSB_PAD_DOWN))
    { //will be TRUE as long as button is pressed
      Serial.print("PSAB_PAD_DOWN held this hard: ");
      Serial.println(GamePadData[PSAB_PAD_DOWN], DEC);
    }

    if (ButtonOn(PSB_GREEN))
    { //will be TRUE as long as button is pressed
      Serial.print("PSAB_GREEN held this hard: ");
      Serial.println(GamePadData[PSAB_GREEN], DEC);
    }
    if (ButtonOn(PSB_RED))
    { //will be TRUE as long as button is pressed
      Serial.print("PSAB_RED held this hard: ");
      Serial.println(GamePadData[PSAB_RED], DEC);
    }
    if (ButtonOn(PSB_BLUE))
    { //will be TRUE as long as button is pressed
      Serial.print("PSAB_BLUE held this hard: ");
      Serial.println(GamePadData[PSAB_BLUE], DEC);

    }

    if (ButtonOn(PSB_PINK))
    { //will be TRUE as long as button is pressed
      Serial.print("PSAB_PINK held this hard: ");
      Serial.println(GamePadData[PSAB_PINK], DEC);
    }

    if (ButtonOn(PSB_L1))
    { //will be TRUE as long as button is pressed
      Serial.print("PSAB_L1 held this hard: ");
      Serial.println(GamePadData[PSAB_L1], DEC);
    }

    if (ButtonOn(PSB_R1))
    { //will be TRUE as long as button is pressed
      Serial.print("PSAB_R1 held this hard: ");
      Serial.println(GamePadData[PSAB_R1], DEC);
    }

    if (ButtonOn(PSB_L2))
    { //will be TRUE as long as button is pressed
      Serial.print("PSAB_L2 held this hard: ");
      Serial.println(GamePadData[PSAB_L2], DEC);
    }

    if (ButtonOn(PSB_R2))
    { //will be TRUE as long as button is pressed
      Serial.print("PSAB_R2 held this hard: ");
      Serial.println(GamePadData[PSAB_R2], DEC);
    }
  */
}
